 /****************************************************************************/
 /*  Program: XV0004C                                                        */
 /*  By:      Alan Campin                                                    */
 /*  Date:    09/08/2005                                                     */
 /*  Purpose: Command processor for EXCSRCCMD. Does setup and                */
 /*            calls EXTSRCCMD extract commands from source member           */
 /*            and then executes one by one sending back any error messages. */
 /*  ------------------------------------------------------------------------*/
 /*  Called by: Command EXCSRCCMD                                            */
 /*--------------------------------------------------------------------------*/
 /* Program Modification Section:                                            */
 /*  By..:                                                                   */
 /*  Date:                                                                   */
 /*  Desc:                                                                   */
 /*                                                                          */
 /****************************************************************************/
             PGM        PARM(&SRCM &SRCFAL &CMDERR &SUBDTA &MESSAGEQ)

             DCL        VAR(&SRCM) TYPE(*CHAR) LEN(10) /* Source +
                          member containing commands. */
             DCL        VAR(&SRCFAL) TYPE(*CHAR) LEN(20) /* Source +
                          file and library. */
             DCL        VAR(&CMDERR) TYPE(*CHAR) LEN(7) /* Action to +
                          take on command error. *ABORT or *BYPASS */
             DCL        VAR(&SUBDTA) TYPE(*CHAR) LEN(128) /* +
                          Substitution data. */
             DCL        VAR(&MESSAGEQ) TYPE(*CHAR) LEN(20) /* +
                          Message Queue Name for Messages or *User +
                          to send to user message queue. */

             DCL        VAR(&DTAQNAME) TYPE(*CHAR) LEN(10)
             DCL        VAR(&DTAQLIB) TYPE(*CHAR) LEN(10) +
                          VALUE('QTEMP')

             DCL        VAR(&ISMSGF) TYPE(*CHAR) LEN(10) +
                          VALUE('XVMSGF')
             DCL        VAR(&ISMSGFLIB) TYPE(*CHAR) LEN(10) +
                          VALUE('*LIBL')

             DCL        VAR(&SRCF) TYPE(*CHAR) LEN(10)
             DCL        VAR(&SRCFL) TYPE(*CHAR) LEN(10)
             DCL        VAR(&MSGQNAM) TYPE(*CHAR) LEN(10)
             DCL        VAR(&MSGQLIB) TYPE(*CHAR) LEN(10)
             DCL        VAR(&USER) TYPE(*CHAR) LEN(10)
             DCL        VAR(&RTNCNT) TYPE(*DEC) LEN(5 0)
             DCL        VAR(&RTNLEN) TYPE(*DEC) LEN(5 0)
             DCL        VAR(&RTNCMD) TYPE(*CHAR) LEN(1024)
             DCL        VAR(&WAIT) TYPE(*DEC) LEN(5 0) VALUE(0)
             DCL        VAR(&CMDLEN) TYPE(*DEC) LEN(15 5)
             DCL        VAR(&SHORTCMD) TYPE(*CHAR) LEN(50)
             DCL        VAR(&FAILCNT) TYPE(*DEC) LEN(5 0)
             DCL        VAR(&OKCNT) TYPE(*DEC) LEN(5 0)
             DCL        VAR(&OKSTG) TYPE(*CHAR) LEN(2)
             DCL        VAR(&FAILSTG) TYPE(*CHAR) LEN(2)
             DCL        VAR(&OK) TYPE(*CHAR) LEN(1)
             DCL        VAR(&TIME) TYPE(*CHAR) LEN(9)

             DCL        VAR(&MSGID) TYPE(*CHAR) LEN(7)
             DCL        VAR(&MSGF) TYPE(*CHAR) LEN(10)
             DCL        VAR(&MSGFLIB) TYPE(*CHAR) LEN(10)
             DCL        VAR(&MSGDTA) TYPE(*CHAR) LEN(512)

/*Unmonitored message handler                                                */

             MONMSG     MSGID(CPF0000 MCH0000) EXEC(GOTO +
                          CMDLBL(HANDLE_ERR))

/*Retrieve source file and library.                                          */

             CHGVAR     VAR(&SRCF) VALUE(%SUBSTRING(&SRCFAL 1 10))
             CHGVAR     VAR(&SRCFL) VALUE(%SUBSTRING(&SRCFAL 11 10))

/*Retrieve the message queue and library if not *User.                       */

             IF         COND(&MESSAGEQ *EQ '*USER') THEN(DO)
               CHGVAR     VAR(&MSGQNAM) VALUE('*USER')
               CHGVAR     VAR(&MSGQLIB) VALUE(' ')
             ENDDO
             ELSE       CMD(DO)
               CHGVAR     VAR(&MSGQNAM) VALUE(%SUBSTRING(&MESSAGEQ 1 10))
               CHGVAR     VAR(&MSGQLIB) VALUE(%SUBSTRING(&MESSAGEQ 11 10))
             ENDDO

/*Clear any message from program message queue.                              */

             RMVMSG     CLEAR(*ALL)

/* Calculate a data queue name based on current time to hundreds. */

             RTVSYSVAL  SYSVAL(QTIME) RTNVAR(&TIME)
             CHGVAR     VAR(&DTAQNAME) VALUE('X' || &TIME)

/*Use EXTSRCCMD to extract list of CL commands in member. Will               */
/*  be return in data queue name specified on command.                       */

             EXTSRCCMD  SRCFILE(&SRCFL/&SRCF) SRCMBR(&SRCM) +
                          SUBDATA(&SUBDTA) +
                          DATAQ(&DTAQLIB/&DTAQNAME) RTNCNT(&RTNCNT)

             IF         COND(&RTNCNT *LE 0) THEN(GOTO CMDLBL(ENDEXC))

/*Clear any message from program message queue.                              */

             RMVMSG     CLEAR(*ALL)

/*If any returned, execute commands in data queue.                           */

EXCCMD:
             IF         COND(&RTNCNT *LE 0) THEN(GOTO CMDLBL(ENDEXC))

             CALL       PGM(QRCVDTAQ) PARM(&DTAQNAME &DTAQLIB +
                          &RTNLEN &RTNCMD &WAIT)

             CHGVAR     VAR(&CMDLEN) VALUE(&RTNLEN)
             CHGVAR     VAR(&OK) VALUE('Y')
             CALL       PGM(QCMDEXC) PARM(&RTNCMD &CMDLEN)
             MONMSG     MSGID(CPF0000) EXEC(DO)
               CHGVAR     VAR(&FAILCNT) VALUE(&FAILCNT +1)
               CHGVAR     VAR(&OK) VALUE('N')
               CHGVAR     VAR(&SHORTCMD) VALUE(%SST(&RTNCMD 1 50))
               SNDPGMMSG  MSGID(UXV0002) MSGF(&ISMSGFLIB/&ISMSGF) +
                            MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL *CAT +
                            &SHORTCMD) TOPGMQ(*SAME)
               IF         COND(&CMDERR *EQ '*ABORT ') THEN(DO)
                 SNDPGMMSG  MSGID(UXV0004) MSGF(&ISMSGFLIB/&ISMSGF) +
                              MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL) +
                              TOPGMQ(*SAME)
                 GOTO       CMDLBL(ENDEXC)
               ENDDO
             ENDDO

             IF         COND(&OK *EQ 'Y') THEN(CHGVAR VAR(&OKCNT) +
                          VALUE(&OKCNT +1))

             CHGVAR     VAR(&RTNCNT) VALUE(&RTNCNT - 1)

             GOTO       CMDLBL(EXCCMD)
ENDEXC:

/*Retrieve user profile for current job.                                     */

             RTVJOBA    CURUSER(&USER)

/*Send time/date stamp to user or named messsage queue.                      */

/*           IF         COND(&MSGQNAM *EQ '*USER') THEN(DO) +
               SNDMSG     MSG(' ') TOUSR(&USER)             +
             ENDDO                                          +
             ELSE       CMD(DO)                             +
               SNDMSG     MSG(' ') TOMSGQ(&MSGQLIB/&MSGQNAM) +
             ENDDO            */

/*Send all stored UXV0001 to UXV0006 messages to user or named message       */
/*queue or else we don't want.                                               */

RCVMSG:
             RCVMSG     MSGDTA(&MSGDTA) MSGID(&MSGID) MSGF(&MSGF) +
                          MSGFLIB(&MSGFLIB)
             MONMSG     MSGID(CPF0000) EXEC(GOTO CMDLBL(ENDRCV))
             IF         COND(&MSGID *EQ ' ') THEN(GOTO CMDLBL(ENDRCV))

             IF         COND(&MSGID *GE 'UXV0001' *AND &MSGID *LE +
                          'UXV0006') THEN(DO)
               IF         COND(&MSGQNAM *EQ '*USER') THEN(DO)
                 SNDPGMMSG  MSGID(&MSGID) MSGF(&MSGFLIB/&MSGF) +
                              MSGDTA(&MSGDTA) TOUSR(&USER)
                 MONMSG     MSGID(CPF0000)
               ENDDO
               ELSE       CMD(DO)
                 SNDPGMMSG  MSGID(&MSGID) MSGF(&MSGFLIB/&MSGF) +
                              MSGDTA(&MSGDTA) TOMSGQ(&MSGQLIB/&MSGQNAM) +
                              MSGTYPE(*INQ)
                 MONMSG     MSGID(CPF0000)
               ENDDO
             ENDDO

             GOTO       CMDLBL(RCVMSG)
ENDRCV:

/*Send count of commands executed or not executed to user.                   */

             IF         COND((&OKCNT *EQ 0) *AND (&FAILCNT *EQ 0)) +
                          THEN(DO)
               IF         COND(&MSGQNAM *EQ '*USER') THEN(DO)
                 SNDPGMMSG  MSGID(UXV0005) MSGF(&ISMSGFLIB/&ISMSGF) +
                              MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL) +
                              TOUSR(&USER)
               ENDDO
               ELSE       CMD(DO)
                 SNDPGMMSG  MSGID(UXV0005) MSGF(&ISMSGFLIB/&ISMSGF) +
                              MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL) +
                               TOMSGQ(&MSGQLIB/&MSGQNAM) MSGTYPE(*INQ)
               ENDDO
             ENDDO
             ELSE       CMD(DO)
               CHGVAR     VAR(%BIN(&OKSTG 1 2)) VALUE(&OKCNT)
               CHGVAR     VAR(%BIN(&FAILSTG 1 2)) VALUE(&FAILCNT)
               IF         COND(&MSGQNAM *EQ '*USER') THEN(DO)
                 SNDPGMMSG  MSGID(UXV0006) MSGF(&ISMSGFLIB/&ISMSGF) +
                              MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL *CAT +
                              %SST(&OKSTG 1 2) *CAT %SST(&FAILSTG 1 2)) +
                              TOUSR(&USER) MSGTYPE(*COMP)
               ENDDO
               ELSE       CMD(DO)
                 SNDPGMMSG  MSGID(UXV0006) MSGF(&ISMSGFLIB/&ISMSGF) +
                              MSGDTA(&SRCM *CAT &SRCF *CAT &SRCFL *CAT +
                              %SST(&OKSTG 1 2) *CAT %SST(&FAILSTG 1 2)) +
                              TOMSGQ(&MSGQLIB/&MSGQNAM) MSGTYPE(*INQ)
               ENDDO
             ENDDO

             GOTO       CMDLBL(ENDPGM)

/*Handle unmonitored errors.                                                 */

HANDLE_ERR:
             RCVMSG     MSGTYPE(*EXCP) MSGDTA(&MSGDTA) MSGID(&MSGID) +
                          MSGF(&MSGF) MSGFLIB(&MSGFLIB)
             MONMSG     MSGID(CPF0000) EXEC(GOTO CMDLBL(END_ABNORM))
             IF         COND(&MSGID *EQ ' ') THEN(GOTO +
                          CMDLBL(END_ABNORM))
             SNDPGMMSG  MSGID(&MSGID) MSGF(&MSGFLIB/&MSGF) +
                          MSGDTA(&MSGDTA) MSGTYPE(*DIAG)
             GOTO       CMDLBL(HANDLE_ERR)
END_ABNORM:
             SNDPGMMSG  MSGID(CPF9898) MSGF(QCPFMSG) MSGDTA('Command +
                          EXCSRCCMD ended abnormally! See previous +
                          messages for reason!') MSGTYPE(*ESCAPE)

/*Get rid of data queue created by EXTSRCCMD.                                */

ENDPGM:
             DLTDTAQ    DTAQ(&DTAQLIB/&DTAQNAME)
             MONMSG     MSGID(CPF0000)

             ENDPGM
